
def get_current_site_absolute(request=None, secure: bool = False):
    from django.contrib.sites.models import Site
    from django.contrib.sites.shortcuts import get_current_site
    from django.conf import settings

    if request:
        current_site = get_current_site(request)
        current_schema = 'https' if request.is_secure() else 'http'
    else:
        current_site = Site.objects.get_current()
        current_schema = 'https' if secure else 'http'
    absolute_domain = getattr(settings, "SITE_BASE_URL", None)
    if absolute_domain is None:
        absolute_domain = current_site.domain

    if '://' not in absolute_domain:
        absolute_domain = '{schema}://{domain}'.format(
            schema=current_schema,
            domain=absolute_domain,
        )
    if absolute_domain[-1] == '/':
        absolute_domain = absolute_domain[:-1]
    return absolute_domain


def make_absolute_url(url: str, secure: bool = None):
    if '://' in url:
        return url
    if secure is None:
        from django.conf import settings
        secure = getattr(settings, 'IS_HTTPS', False)
    current_domain = get_current_site_absolute(secure=secure)
    return current_domain + url



import os

from django.core.exceptions import ValidationError
from django.utils.deconstruct import deconstructible


def get_available_svg_image_extensions():
    try:
        from PIL import Image
    except ImportError:
        return []
    else:
        Image.init()
        list_exstentions = [ext.lower()[1:] for ext in Image.EXTENSION.keys()]
        list_exstentions.append("svg")
        return list_exstentions


def validate_svg(f):
    try:
        import xml.etree.cElementTree as et

        # Find "start" word in file and get "tag" from there
        f.seek(0)
        tag = None
        try:
            for event, el in et.iterparse(f, ("start",)):
                tag = el.tag
                break
        except et.ParseError:
            pass

        # Check that this "tag" is correct
        if tag != "{http://www.w3.org/2000/svg}svg":
            raise ValidationError("Uploaded file is not an image or SVG file.")

        # Do not forget to "reset" file
        f.seek(0)

        return f
    except ImportError:
        raise ValidationError(
            "validate_svg requires xml package to work, please install it"
        )


@deconstructible
class UploadPath(object):
    """
    big_snapshot = models.ImageField(_('Big Snapshot'), upload_to=UploadPath('big_snapshot'), blank=True)
    """

    def __init__(self, attr_name):
        self.attr_name = attr_name

    def __call__(self, instance, filename):
        return self.get_upload_path(instance, filename)

    def get_upload_path(self, instance, filename):
        return os.path.join(
            "uploads",
            instance._meta.app_label.lower(),
            instance.__class__.__name__.lower(),
            self.attr_name.lower(),
            os.path.basename(filename),
        )


@deconstructible
class UploadPathWithID(UploadPath):
    def get_upload_path(self, instance, filename):
        return os.path.join(
            "uploads",
            instance._meta.app_label.lower(),
            instance.__class__.__name__.lower(),
            self.attr_name.lower(),
            str(instance.id),
            os.path.basename(filename),
        )



import logging
import re

from django.contrib.admin.widgets import AdminFileWidget
from django.utils.safestring import mark_safe
from django.utils.translation import gettext as _

from os import path

from easy_thumbnails.exceptions import InvalidImageFormatError

from file_field_utils.utils import make_absolute_url

logger = logging.getLogger("django-file-field-utils")

from easy_thumbnails.files import get_thumbnailer


class ConfigurableImageWidget(AdminFileWidget):
    """
    use it whit this statement if your admin inherits from ConfigurableWidgetsMixinAdmin
        dbfield_overrides = {'img':{'widget':ConfigurableImageWidget({'width':80,'height':80})}}
    """
    default_width = 60
    default_height = 60
    default_crop = "smart"

    class Media:
        css = {
            'all': ('fhcore/admin/css/admin_form_overrides.css',)
        }

    def __init__(self, attrs=None):
        if attrs is not None:
            for k in ['width', 'height', 'crop']:
                try:
                    setattr(self, k, attrs.pop(k))
                except KeyError:
                    setattr(self, k, None)
        super(ConfigurableImageWidget, self).__init__
        if attrs is not None:
            self.attrs = attrs.copy()
        else:
            self.attrs = {}

    def get_width(self):
        return getattr(self, 'width', self.default_width)

    def get_height(self):
        return getattr(self, 'height', self.default_height)

    def get_crop(self):
        return getattr(self, 'crop', self.default_crop)

    def get_image_opts(self):
        return {'size': (self.get_width(), self.get_height()),
                'crop': self.get_crop()}

    def get_image_html(self, value, thumbnail_url):
        help_text = "" if not hasattr(self, "help_text") else self.help_text
        output_html = f'<div class="box img-preview" style="float:left;margin-right:5px;">' \
                        f'<h4 style="text-align:center;color:#fff;background:#9AB8D6;">{_("Preview")}</h4>' \
                            f'<a target="_blank" href="{value.url}">' \
                                f'<img src="{thumbnail_url}" alt="{path.split(value.url)[-1]}" />' \
                            f'</a>' \
                      f'</div>' \
                      f'<div class="help">{help_text}</div>{_("Change:")} '
        return output_html

    def is_svg_image(self, file_path):
        import re
        from urllib.request import urlopen

        SVG_R = r'(?:<\?xml\b[^>]*>[^<]*)?(?:<!--.*?-->[^<]*)*(?:<svg|<!DOCTYPE svg)\b'
        SVG_RE = re.compile(SVG_R, re.DOTALL)

        f = urlopen(f"{make_absolute_url(file_path)}")

        file_contents = f.read().decode('latin_1')  # avoid any conversion exception

        is_svg = SVG_RE.match(file_contents) is not None

        return is_svg

    def render(self, name, value, attrs=None, **kwargs):
        output = []
        if value and hasattr(value, "url"):
            try:
                if not self.is_svg_image(value.url):
                    if hasattr(self, 'thumbnail') and self.thumbnail:
                        thumbnail = self.thumbnail
                    else:
                        realname = re.sub(r'^(\w+)-(\d+)-', '', name)
                        image_preview = getattr(value.instance, realname)
                        opts = self.get_image_opts()
                        thumbnail = get_thumbnailer(image_preview).get_thumbnail(opts)
                    thumbnail_url = thumbnail.url
                else:
                    thumbnail_url = value.url
                output.append(self.get_image_html(value, thumbnail_url))

            except IOError:
                pass
            except InvalidImageFormatError:
                logger.info("File con formato non previsto")
            except Exception as ex:
                raise
        output.append(super(AdminFileWidget, self).render(name, value, attrs))
        return mark_safe(u''.join(output))



from django.db import models

from file_field_utils.db.forms import SVGAndImageFieldForm


class SVGAndImageField(models.ImageField):
    def formfield(self, **kwargs):
        defaults = {'form_class': SVGAndImageFieldForm}
        defaults.update(kwargs)
        return super().formfield(**defaults)



from django.core.exceptions import ValidationError
from django.core.validators import FileExtensionValidator
from django import forms

from file_field_utils.db.utils.backend import get_available_svg_image_extensions, validate_svg


validate_svg_image_file_extension = FileExtensionValidator(
    allowed_extensions=get_available_svg_image_extensions(),
)


class SVGAndImageFieldForm(forms.ImageField):
    default_validators = [validate_svg_image_file_extension]

    def to_python(self, data):
        try:
            f = super().to_python(data)
        except ValidationError:
            return validate_svg(data)

        return f



import os

from django.conf import settings

PROJECT_PATH_DEFAULT, _ = os.path.split(os.path.split(os.path.realpath(__file__))[0])
PROJECT_PATH = getattr(settings, "PROJECT_PATH", PROJECT_PATH_DEFAULT)

PRIVATE_ROOT_DEFAULT = os.path.join(PROJECT_PATH, "privatemedia")
PRIVATE_ROOT = getattr(settings, "PRIVATE_ROOT", PRIVATE_ROOT_DEFAULT)

PRIVATE_URL_DEFAULT = "/privatemedia/"
PRIVATE_URL = getattr(settings, "PRIVATE_URL", PRIVATE_URL_DEFAULT)
